/*
 * Copyright (C) 2013 by the University of Southern California
 * $Id$
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <iostream>
#include <getopt.h>
#include <signal.h>
#include <sys/stat.h> 	//S_ISDIR
#include <cstring>
#include "scramble_crypt.h"
#include "base64_util.hh"
#include "aes_util.hh"
#include "anon.hh"
using namespace std;

//the key and initialization vector
unsigned char *enc_key = NULL;
unsigned char *enc_iv = NULL;
int key_len = 0;
int iv_len = 0;

/*
	output usage
*/
void usage(const char *comm){
	cerr<<" Usage:\n\t"<<
		comm << " [-r] [-k KEYFILE]\n"
		"\t-r			reverse-mode, decrypting domain names\n"
		"\t-k KEYFILE		specify the key file.\n";
	exit(1);
}

/*
	signal handler
*/
void sig_handler (int sig)
{
	cout<<"user kills the program."<<endl;
	exit(0);
}

/*
	get the enc key from the given keyfile
*/
void get_enc_key (string &keyFile)
{
	scramble_state_t scr_st;	//for getting the key 
	const char *keyFileName = (const char *)keyFile.c_str();
	/*if (scramble_init_from_file(keyFileName, SCRAMBLE_BLOWFISH, SCRAMBLE_BLOWFISH, NULL) < 0) {
		cerr << "error reading keyfile " << keyFileName << endl;
		exit(-1);
	}*/
	
	//get the key for hashing
	const int max_blk_length = 32;//scramble_crypt.cc: define MAX_BLK_LENGTH	32
	
	u_char ppad[max_blk_length];
	u_char kkey[max_blk_length];
	u_char mmac[max_blk_length];
	u_char iiv[max_blk_length];

	scr_st.iv	= iiv;
	scr_st.pad 	= ppad;	
	scr_st.key 	= kkey;	
	scr_st.mac 	= mmac;
		
	scr_st.ivlen	= max_blk_length;
	scr_st.plen 	= max_blk_length;
	scr_st.klen 	= max_blk_length;
	scr_st.mlen 	= max_blk_length;
	
	if (scramble_readstate(keyFileName, &scr_st) < 0) {
		cerr << "[ERROR] scramble_readstate" << endl;
		exit(-1);
	}		
	key_len = scr_st.klen; 
	enc_key = new unsigned char[key_len];
	memcpy(enc_key, scr_st.key, key_len);	
		
	iv_len = scr_st.ivlen;
	enc_iv = new unsigned char[iv_len];
	memcpy(enc_iv, scr_st.iv, iv_len);
	
	if (iv_len == 0) {
		cerr << "[ERROR] iv_len == 0, abort" << endl;
		exit(-1);
	}
	if (key_len == 0) {
		cerr << "[ERROR] key_len == 0, abort." << endl;
		exit(-1);
	}
}

/*
	check path and output errors, input is a file, output is a directory
*/
void check_path(char *f, bool is_dir)
{
	if (strlen(f) > 0) {
    		struct stat statbuf;
		if (stat(f, &statbuf) == -1) {  //get the property of the file
			cerr << "[ERROR]:" << f << " ";
			perror("cannot be opened");
			exit(-1);
        	}
        	if (is_dir) {	//should be a directory
			if(!S_ISDIR(statbuf.st_mode)) {
				cerr << "[ERROR]:" << f << " is a not directory" << endl;
				exit(-1);
			}        		
        	} else {	//should be a file
			if(S_ISDIR(statbuf.st_mode)) {
				cerr << "[ERROR]:" << f << " is a directory" << endl;
				exit(-1);
			} 
        	}
	} else {
		cerr << "[ERROR] \"" << f <<" \":path is not valid." << endl;
		exit(-1);
	}
}

int main (int argc, char *argv[]) {
	const char *comm = argv[0];	//command
	signal (SIGQUIT, sig_handler);//signal
	signal (SIGINT, sig_handler);
	string keyfn;
	int reverse_mode = -1;
	int opt = -1;
	while ((opt = getopt(argc, argv, "k:r")) != EOF) {
		switch(opt) {
			case 'k':
				keyfn = optarg;
				break;
			case 'r':
				reverse_mode = 1;
				break;              		
                    	default:
                    		usage(comm);
		}
	}
        
        //check the commandline result
        if (keyfn.length() == 0) {
        	cerr<<"[ERROR] [-k KEYFILE] is required"<<endl;
        	exit(-1);
        }
	check_path((char *)keyfn.c_str(), false);	//this key file must exist
	get_enc_key (keyfn);
	
	string line;
	string result = "";
	while (getline(cin, line)) {
		anon_name((char *)line.c_str(), enc_key, key_len, enc_iv, iv_len, result, (reverse_mode == 1));
		cout<< result <<endl;
	}
	return 0;
}
